==> forestry_1.12.2-5.8.0.255-sources.jar: forestry/farming/logic/farmables/FarmableAgingCrop.java <==
package forestry.farming.logic.farmables;

import com.google.common.base.Preconditions;

import javax.annotation.Nullable;

import net.minecraft.block.Block;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import forestry.api.farming.ICrop;
import forestry.api.farming.IFarmable;
import forestry.core.utils.BlockUtil;
import forestry.farming.logic.crops.CropDestroy;

/**
 * For blocks that are harvestable once they are a certain age.
 */
public class FarmableAgingCrop implements IFarmable {
	protected final ItemStack germling;
	protected final Block cropBlock;
	protected final IProperty<Integer> ageProperty;
	protected final int minHarvestAge;
	@Nullable
	protected final Integer replantAge;

	public FarmableAgingCrop(ItemStack germling, Block cropBlock, IProperty<Integer> ageProperty, int minHarvestAge) {
		this(germling, cropBlock, ageProperty, minHarvestAge, null);
	}

	public FarmableAgingCrop(ItemStack germling, Block cropBlock, IProperty<Integer> ageProperty, int minHarvestAge, @Nullable Integer replantAge) {
		Preconditions.checkNotNull(germling);
		Preconditions.checkNotNull(cropBlock);
		Preconditions.checkNotNull(ageProperty);

		this.germling = germling;
		this.cropBlock = cropBlock;
		this.ageProperty = ageProperty;
		this.minHarvestAge = minHarvestAge;
		this.replantAge = replantAge;
	}

	@Override
	public boolean isSaplingAt(World world, BlockPos pos, IBlockState blockState) {
		return blockState.func_177230_c() == cropBlock && blockState.func_177229_b(ageProperty) <= minHarvestAge;
	}

	@Override
	@Nullable
	public ICrop getCropAt(World world, BlockPos pos, IBlockState blockState) {
		if (blockState.func_177230_c() != cropBlock) {
			return null;
		}

		if (blockState.func_177229_b(ageProperty) < minHarvestAge) {
			return null;
		}

		IBlockState replantState = getReplantState(blockState);
		return new CropDestroy(world, blockState, pos, replantState, germling);
	}

	@Nullable
	protected IBlockState getReplantState(IBlockState blockState) {
		if (replantAge == null) {
			return null;
		}
		return blockState.func_177226_a(ageProperty, replantAge);
	}

	@Override
	public boolean isGermling(ItemStack itemstack) {
		return ItemStack.func_179545_c(germling, itemstack);
	}

	@Override
	public boolean plantSaplingAt(EntityPlayer player, ItemStack germling, World world, BlockPos pos) {
		IBlockState plantedState = cropBlock.func_176223_P().func_177226_a(ageProperty, 0);
		return BlockUtil.setBlockWithPlaceSound(world, pos, plantedState);
	}

	@Override
	public boolean isWindfall(ItemStack itemstack) {
		return false;
	}
}

==> forestry_1.12.2-5.8.2.378-sources.jar: forestry/farming/logic/farmables/FarmableAgingCrop.java <==
package forestry.farming.logic.farmables;

import com.google.common.base.Preconditions;

import javax.annotation.Nullable;

import net.minecraft.block.Block;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import forestry.api.farming.ICrop;
import forestry.api.farming.IFarmable;
import forestry.api.farming.IFarmableInfo;
import forestry.core.utils.BlockUtil;
import forestry.farming.logic.crops.CropDestroy;

/**
 * For blocks that are harvestable once they are a certain age.
 */
public class FarmableAgingCrop implements IFarmable {
	protected final ItemStack germling;
	protected final Block cropBlock;
	protected final IProperty<Integer> ageProperty;
	protected final int minHarvestAge;
	@Nullable
	protected final Integer replantAge;
	protected final ItemStack[] products;

	public FarmableAgingCrop(ItemStack germling, Block cropBlock, IProperty<Integer> ageProperty, int minHarvestAge) {
		this(germling, cropBlock, new ItemStack[0], ageProperty, minHarvestAge, null);
	}

	public FarmableAgingCrop(ItemStack germling, Block cropBlock, IProperty<Integer> ageProperty, int minHarvestAge, @Nullable Integer replantAge) {
		this(germling, cropBlock, new ItemStack[0], ageProperty, minHarvestAge, replantAge);
	}

	public FarmableAgingCrop(ItemStack germling, Block cropBlock, ItemStack product, IProperty<Integer> ageProperty, int minHarvestAge) {
		this(germling, cropBlock, new ItemStack[]{product}, ageProperty, minHarvestAge, null);
	}

	public FarmableAgingCrop(ItemStack germling, Block cropBlock, ItemStack product, IProperty<Integer> ageProperty, int minHarvestAge, @Nullable Integer replantAge) {
		this(germling, cropBlock, new ItemStack[]{product}, ageProperty, minHarvestAge, replantAge);
	}

	public FarmableAgingCrop(ItemStack germling, Block cropBlock, ItemStack[] products, IProperty<Integer> ageProperty, int minHarvestAge) {
		this(germling, cropBlock, products, ageProperty, minHarvestAge, null);
	}

	public FarmableAgingCrop(ItemStack germling, Block cropBlock, ItemStack[] products, IProperty<Integer> ageProperty, int minHarvestAge, @Nullable Integer replantAge) {
		Preconditions.checkNotNull(germling);
		Preconditions.checkNotNull(cropBlock);
		Preconditions.checkNotNull(ageProperty);
		Preconditions.checkNotNull(products);

		this.germling = germling;
		this.cropBlock = cropBlock;
		this.ageProperty = ageProperty;
		this.minHarvestAge = minHarvestAge;
		this.replantAge = replantAge;
		this.products = products;
	}

	@Override
	public boolean isSaplingAt(World world, BlockPos pos, IBlockState blockState) {
		return blockState.func_177230_c() == cropBlock && blockState.func_177229_b(ageProperty) <= minHarvestAge;
	}

	@Override
	@Nullable
	public ICrop getCropAt(World world, BlockPos pos, IBlockState blockState) {
		if (blockState.func_177230_c() != cropBlock) {
			return null;
		}

		if (blockState.func_177229_b(ageProperty) < minHarvestAge) {
			return null;
		}

		IBlockState replantState = getReplantState(blockState);
		return new CropDestroy(world, blockState, pos, replantState, germling);
	}

	@Nullable
	protected IBlockState getReplantState(IBlockState blockState) {
		if (replantAge == null) {
			return null;
		}
		return blockState.func_177226_a(ageProperty, replantAge);
	}

	@Override
	public boolean isGermling(ItemStack itemstack) {
		return ItemStack.func_179545_c(germling, itemstack);
	}

	@Override
	public void addInformation(IFarmableInfo info) {
		info.addProducts(products);
		info.addGermlings(germling);
	}

	@Override
	public boolean plantSaplingAt(EntityPlayer player, ItemStack germling, World world, BlockPos pos) {
		IBlockState plantedState = cropBlock.func_176223_P().func_177226_a(ageProperty, 0);
		return BlockUtil.setBlockWithPlaceSound(world, pos, plantedState);
	}

	@Override
	public boolean isWindfall(ItemStack itemstack) {
		return false;
	}
}
